import time,json
from tablestore import *
from tablestore.error import *
from tablestore.metadata import *
from tablestore.retry import *
from tablestore.metadata import *
from tablestore.plainbuffer import *
from tablestore.protobuf import *
from tablestore.protobuf.table_store_pb2 import IndexMeta,PrimaryKeySchema
from tablestore.client import OTSClient

import json

from config import *

# 配置需填入您的实例名称、访问 ID 和密钥以及表名
instance_name = "sd-demo-data"

table_name = 'ots_sd_task_list'
region = 'cn-shanghai'

# 创建连接

end_point = 'https://{}.{}.tablestore.aliyuncs.com'.format(instance_name, region)

# 创建OTS客户端
ots_client = OTSClient(
    end_point,
    access_id,
    access_key,
    instance_name,
)

def create_table():
    # 定义表格模式和索引
    # PK, user_id, SD实例ID, 任务ID，模型名称，任务类型，提示词，请求参数，创建人，创建时间，更新时间，图片列表，请求结果，任务状态，完成时间

    # 创建表格
    try:
        schema_of_primary_key = [
            ('pk', 'STRING'), # PrimaryKeySchema
            ('user_id', 'STRING'),
            # sd实例ID
            ('sd_instance_id', 'STRING'), # PrimaryKeySchema
            # 任务ID
            ('task_id', 'STRING')
        ]
        defined_columns = [
            ('model_name', 'STRING'), 
            ('task_type', 'STRING'), 
            ('prompt', 'STRING'), 
            ('request_params', 'STRING'),
            ('creator', 'STRING'), 
            ('create_time', 'INTEGER'), 
            ('update_time', 'INTEGER'), 
            ('image_list', 'STRING'), 
            ('result', 'STRING'), 
            ('task_status', 'STRING'), 
            ('complete_time', 'INTEGER')
        ]
        table_meta = TableMeta(table_name, schema_of_primary_key, defined_columns)
        table_option = TableOptions(-1, 1)
        reserved_throughput = ReservedThroughput(CapacityUnit(0, 0))
        secondary_indexes = [
            SecondaryIndexMeta('index1', ['pk', 'user_id', 'sd_instance_id', 'task_id'], [ 'model_name', 'task_type', 'task_status']),
            ]
        ots_client.create_table(table_meta, table_option, reserved_throughput, secondary_indexes)                    

        print("Table is created.")
        return True
    except OTSServiceError as e:
        if e.get_error_code() == 'OTSObjectAlreadyExist':
            print("Table already exists.")
        else:
            print("Create table failed.%s" % repr(e))
    return False

def insert_row(pk, user_id, sd_instance_id, task_id, prompt, image_list, model_name='', task_type='txt2img', request_params='', creator='', task_status='init'):
    # 定义数据行
    now = int(time.time())
    primary_key = [('pk', pk), ('user_id', user_id), ('sd_instance_id', sd_instance_id), ('task_id', task_id)]
    attribute_columns = [('model_name', model_name), ('task_type', task_type), 
                        ('prompt', prompt), 
                        ('request_params', request_params), ('creator', creator),
                        ('create_time', now), ('update_time', now), 
                        ('image_list', json.dumps(image_list)), 
                        ('result', ''), 
                        ('task_status', 'completed'), ('complete_time', 0)]
    row = Row(primary_key, attribute_columns)

    # 将数据行插入到Tablestore表中
    condition = Condition(RowExistenceExpectation.EXPECT_NOT_EXIST)
    try:
        consumed, return_row = ots_client.put_row(table_name, row, condition)
        print('Put row succeeded.')
    except Exception as e:
        e.print_stack()
        print('Put row failed:', repr(e))

# entry point
if __name__ == '__main__':
    import uuid
    create_table()
    insert_row('hello', 'world', 'test_instance', str(uuid.uuid4()), 'test prompt', ['abcd.png'], model_name='test_model', task_type='txt2img', request_params=json.dumps({"abc":"1234"}, ensure_ascii=False), creator= 'Tom', task_status='init')